@@ -1,11 +1,11 @@
-/* This code is based on the jpeg saving code from gdk-pixbuf. Full copyright 
+/* This code is based on the jpeg saving code from gdk-pixbuf. Full copyright
  * notice is given in the following:
  */
 /* GdkPixbuf library - JPEG image loader
  *
  * Copyright (C) 1999 Michael Zucchi
  * Copyright (C) 1999 The Free Software Foundation
- * 
+ *
  * Progressive loading code Copyright (C) 1999 Red Hat, Inc.
  *
  * Authors: Michael Zucchi <zucchi@zedzone.mmc.com.au>
@@ -49,7 +49,6 @@
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs.h>
 #if HAVE_EXIF
 #include <libexif/exif-data.h>
 #endif
@@ -71,7 +70,7 @@ struct error_handler_data {
 	struct jpeg_error_mgr pub;
 	sigjmp_buf setjmp_buffer;
         GError **error;
-	const char *filename;
+	char *filename;
 };
 
 
@@ -80,9 +79,9 @@ fatal_error_handler (j_common_ptr cinfo)
 {
 	struct error_handler_data *errmgr;
         char buffer[JMSG_LENGTH_MAX];
-        
+
 	errmgr = (struct error_handler_data *) cinfo->err;
-        
+
         /* Create the message */
         (* cinfo->err->format_message) (cinfo, buffer);
 
@@ -124,7 +123,7 @@ init_transform_info (EogImage *image, jpeg_transform_info *info)
 	priv = image->priv;
 
 	if (priv->trans != NULL && priv->trans_autorotate != NULL) {
-		composition = eog_transform_compose (priv->trans, 
+		composition = eog_transform_compose (priv->trans,
 						     priv->trans_autorotate);
 	} else if (priv->trans != NULL) {
 		composition = g_object_ref (priv->trans);
@@ -137,13 +136,13 @@ init_transform_info (EogImage *image, jpeg_transform_info *info)
 
 		switch (transformation) {
 		case EOG_TRANSFORM_ROT_90:
-			trans_code = JXFORM_ROT_90; 
+			trans_code = JXFORM_ROT_90;
 			break;
 		case EOG_TRANSFORM_ROT_270:
-			trans_code = JXFORM_ROT_270; 
+			trans_code = JXFORM_ROT_270;
 			break;
 		case EOG_TRANSFORM_ROT_180:
-			trans_code = JXFORM_ROT_180; 
+			trans_code = JXFORM_ROT_180;
 			break;
 		case EOG_TRANSFORM_FLIP_HORIZONTAL:
 			trans_code = JXFORM_FLIP_H;
@@ -165,13 +164,13 @@ init_transform_info (EogImage *image, jpeg_transform_info *info)
 }
 
 static gboolean
-_save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source, 
+_save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 		    EogImageSaveInfo *target, GError **error)
 {
 	struct jpeg_decompress_struct  srcinfo;
 	struct jpeg_compress_struct    dstinfo;
 	struct error_handler_data      jsrcerr, jdsterr;
-	jpeg_transform_info            transformoption; 
+	jpeg_transform_info            transformoption;
 	jvirt_barray_ptr              *src_coef_arrays;
 	jvirt_barray_ptr              *dst_coef_arrays;
 	FILE                          *output_file;
@@ -180,15 +179,15 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	gchar                          *infile_uri;
 
 	g_return_val_if_fail (EOG_IS_IMAGE (image), FALSE);
-	g_return_val_if_fail (EOG_IMAGE (image)->priv->uri != NULL, FALSE);
+	g_return_val_if_fail (EOG_IMAGE (image)->priv->file != NULL, FALSE);
 
 	priv = image->priv;
 
 	init_transform_info (image, &transformoption);
-	
-	/* Initialize the JPEG decompression object with default error 
+
+	/* Initialize the JPEG decompression object with default error
 	 * handling. */
-	jsrcerr.filename = gnome_vfs_uri_get_path (priv->uri);
+	jsrcerr.filename = g_file_get_path (priv->file);
 	srcinfo.err = jpeg_std_error (&(jsrcerr.pub));
 	jsrcerr.pub.error_exit = fatal_error_handler;
 	jsrcerr.pub.output_message = output_message_handler;
@@ -196,16 +195,16 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 
 	jpeg_create_decompress (&srcinfo);
 
-	/* Initialize the JPEG compression object with default error 
+	/* Initialize the JPEG compression object with default error
 	 * handling. */
-	jdsterr.filename = file;
+	jdsterr.filename = (char *) file;
 	dstinfo.err = jpeg_std_error (&(jdsterr.pub));
 	jdsterr.pub.error_exit = fatal_error_handler;
 	jdsterr.pub.output_message = output_message_handler;
 	jdsterr.error = error;
 
 	jpeg_create_compress (&dstinfo);
-	
+
 	dstinfo.err->trace_level = 0;
 	dstinfo.arith_code = FALSE;
 	dstinfo.optimize_coding = FALSE;
@@ -214,11 +213,12 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	srcinfo.mem->max_memory_to_use = dstinfo.mem->max_memory_to_use;
 
 	/* Open the output file. */
-	/* FIXME: Make this a GnomeVFSURI aware input manager */
-	infile_uri = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (priv->uri), NULL);
+	/* FIXME: Make this a GIO aware input manager */
+	infile_uri = g_file_get_path (priv->file);
 	input_file = fopen (infile_uri, "rb");
 	if (input_file == NULL) {
 		g_warning ("Input file not openable: %s\n", infile_uri);
+		g_free (jsrcerr.filename);
 		g_free (infile_uri);
 		return FALSE;
 	}
@@ -228,6 +228,7 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	if (output_file == NULL) {
 		g_warning ("Output file not openable: %s\n", file);
 		fclose (input_file);
+		g_free (jsrcerr.filename);
 		return FALSE;
 	}
 
@@ -236,6 +237,7 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 		fclose (input_file);
 		jpeg_destroy_compress (&dstinfo);
 		jpeg_destroy_decompress (&srcinfo);
+		g_free (jsrcerr.filename);
 		return FALSE;
 	}
 
@@ -244,6 +246,7 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 		fclose (input_file);
 		jpeg_destroy_compress (&dstinfo);
 		jpeg_destroy_decompress (&srcinfo);
+		g_free (jsrcerr.filename);
 		return FALSE;
 	}
 
@@ -270,7 +273,7 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	/* Adjust destination parameters if required by transform options;
 	 * also find out which set of coefficient arrays will hold the output.
 	 */
-	dst_coef_arrays = jtransform_adjust_parameters (&srcinfo, 
+	dst_coef_arrays = jtransform_adjust_parameters (&srcinfo,
 							&dstinfo,
 							src_coef_arrays,
 							&transformoption);
@@ -301,12 +304,12 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 #endif
 	/* FIXME: Consider IPTC data too */
 
-	/* Copy to the output file any extra markers that we want to 
+	/* Copy to the output file any extra markers that we want to
 	 * preserve */
 	jcopy_markers_execute (&srcinfo, &dstinfo, JCOPYOPT_DEFAULT);
 
 	/* Execute image transformation, if any */
-	jtransform_execute_transformation (&srcinfo, 
+	jtransform_execute_transformation (&srcinfo,
 					   &dstinfo,
 					   src_coef_arrays,
 					   &transformoption);
@@ -316,16 +319,17 @@ _save_jpeg_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	jpeg_destroy_compress (&dstinfo);
 	(void) jpeg_finish_decompress (&srcinfo);
 	jpeg_destroy_decompress (&srcinfo);
+	g_free (jsrcerr.filename);
 
 	/* Close files */
 	fclose (input_file);
 	fclose (output_file);
 
-	return TRUE;	
+	return TRUE;
 }
 
 static gboolean
-_save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source, 
+_save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 		   EogImageSaveInfo *target, GError **error)
 {
 	EogImagePrivate *priv;
@@ -342,13 +346,13 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	int rowstride = 0;
 	FILE *outfile;
 	struct error_handler_data jerr;
-	
+
 	g_return_val_if_fail (EOG_IS_IMAGE (image), FALSE);
 	g_return_val_if_fail (EOG_IMAGE (image)->priv->image != NULL, FALSE);
-	
+
 	priv = image->priv;
 	pixbuf = priv->image;
-	
+
 	outfile = fopen (file, "wb");
 	if (outfile == NULL) {
 		g_set_error (error,             /* FIXME: Better error message */
@@ -362,11 +366,11 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 	w = gdk_pixbuf_get_width (pixbuf);
 	h = gdk_pixbuf_get_height (pixbuf);
-	
+
 	/* no image data? abort */
 	pixels = gdk_pixbuf_get_pixels (pixbuf);
 	g_return_val_if_fail (pixels != NULL, FALSE);
-	
+
 	/* allocate a small buffer to convert image data */
 	buf = g_try_malloc (w * 3 * sizeof (guchar));
 	if (!buf) {
@@ -374,11 +378,12 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 			     GDK_PIXBUF_ERROR,
 			     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
 			     _("Couldn't allocate memory for loading JPEG file"));
+		fclose (outfile);
 		return FALSE;
 	}
-	
-	/* set up error handling */	
-	jerr.filename = file;
+
+	/* set up error handling */
+	jerr.filename = (char *) file;
 	cinfo.err = jpeg_std_error (&(jerr.pub));
 	jerr.pub.error_exit = fatal_error_handler;
 	jerr.pub.output_message = output_message_handler;
@@ -389,11 +394,12 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	jpeg_stdio_dest (&cinfo, outfile);
 	cinfo.image_width      = w;
 	cinfo.image_height     = h;
-	cinfo.input_components = 3; 
+	cinfo.input_components = 3;
 	cinfo.in_color_space   = JCS_RGB;
 
 	/* error exit routine */
 	if (sigsetjmp (jerr.setjmp_buffer, 1)) {
+		g_free (buf);
 		fclose (outfile);
 		jpeg_destroy_compress (&cinfo);
 		return FALSE;
@@ -403,12 +409,12 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	if (target != NULL && target->jpeg_quality >= 0.0) {
 		quality = (int) MIN (target->jpeg_quality, 1.0) * 100;
 	}
-	
+
 	/* set up jepg compression parameters */
 	jpeg_set_defaults (&cinfo);
 	jpeg_set_quality (&cinfo, quality, TRUE);
 	jpeg_start_compress (&cinfo, TRUE);
-	
+
 	/* write EXIF/IPTC data explicitly */
 #if HAVE_EXIF
 	/* exif_chunk and exif are mutally exclusvie, this is what we assure here */
@@ -417,7 +423,7 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	{
 		unsigned char *exif_buf;
 		unsigned int   exif_buf_len;
-		
+
 		exif_data_save_data (priv->exif, &exif_buf, &exif_buf_len);
 		jpeg_write_marker (&cinfo, 0xe1, exif_buf, exif_buf_len);
 		g_free (exif_buf);
@@ -437,15 +443,15 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 		/* convert scanline from ARGB to RGB packed */
 		for (j = 0; j < w; j++)
 			memcpy (&(buf[j*3]), &(ptr[i*rowstride + j*(rowstride/w)]), 3);
-		
+
 		/* write scanline */
 		jbuf = (JSAMPROW *)(&buf);
 		jpeg_write_scanlines (&cinfo, jbuf, 1);
 		i++;
 		y++;
-		
+
 	}
-	
+
 	/* finish off */
 	jpeg_finish_compress (&cinfo);
 	jpeg_destroy_compress(&cinfo);
@@ -456,8 +462,8 @@ _save_any_as_jpeg (EogImage *image, const char *file, EogImageSaveInfo *source,
 	return TRUE;
 }
 
-gboolean 
-eog_image_jpeg_save_file (EogImage *image, const char *file, 
+gboolean
+eog_image_jpeg_save_file (EogImage *image, const char *file,
 			  EogImageSaveInfo *source, EogImageSaveInfo *target,
 			  GError **error)
 {
